<?php
namespace Financeiro\Logic\ContasReceber\Boleto;

use \Financeiro\Entities\Boletreceber;
use \Financeiro\Entities\Contareceber;
use \Financeiro\Entities\Ajbolrec;
use \Financeiro\Logic\ContasReceber\Boleto\EmissaoBoleto\EmissaoBoleto;
use \Financeiro\Logic\ContasReceber\Boleto\EmissaoBoleto\ValidacaoEmissaoBoleto;
use \Financeiro\Logic\ContasReceber\Boleto\Exception\ExcecaoProrrogacao;
use Geral\Util\Ambiente;
use \Geral\Util\DateUtil;
use \Financeiro\Logic\AtualizaFluxodeCaixa\FluxodeCaixa;
use \Financeiro\Entities\Ajctreceber;
use \Faturamento\Entities\Tipoajreceber;
use \Financeiro\Logic\ContasReceber\Boleto\Exception\ExcecaoCnab;
use \Geral\Util\NovoCodigo;
use \Financeiro\Repositories\Adiantareceb;
use \Financeiro\Entities\Cnabrec;
use \Fiscal\Entities\Notafiscal;
use \Financeiro\Entities\GeralRec;
use \Financeiro\Entities\Ratctreceber;
use \Indeterminado\Entities\Devolucaorec;
use \Financeiro\Logic\ContasReceber\Boleto\Exception\ExcecaoCancelamento;
use Setforms2\Runtime\Helper\Procedure;
use Setforms2\Runtime\Helper\Procedure\Param;
use \Financeiro\Entities\Tprecboleto;
/**
 *  *
 *
 *
 * Created by JetBrains PhpStorm.
 * User: paulopereira
 * Date: 09/04/13
 * Time: 11:14
 * To change this template use File | Settings | File Templates.
 */
class OperacoesBoleto
{
    const TAMANHO_NRLANBOLREC = 10;
    const PREFIXO_NOVOCODIGO_BOLREC    = 'BOLETRECEBER';
    const STATUS_BOLETO_PENDENTE  = 'A';
    const ID_CNAB_N_ENVIADO = '00';
    const IDBOLRECALTER_DEFAULT = '1';
    const IDFORMCADBREC_DEFAULT = '1';
    const ID_AJ_BOL_COMP = '4';
    const PERMITE_ALTERACAO_BOLETO = '2';
    const NAO_PERMITE_ALETRACAO_BOLETO = '1';
    const BOLETO_ALTERADO = '3';




    /**
     * Alteracao dos dados do boleto.
     *
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $cdreducont
     * @param $cdtpbaixarec
     * @param $cdcartcobbol
     * @param $idcartcobrec
     * @param $idimprbolbol
     * @param $cdins1cobbol
     * @param $cdins2cobbol
     * @param $qtdiaprotbol
     * @param $cddiaprotbol
     * @param $idformcadbre
     * @param $vrpemulbrec
     * @param $vrmultabrec
     * @param $vrpemorbrec
     * @param $vrmordiabrec
     * @throws \Exception
     */
    public function alteraBoleto($nrorg, $cdfilial, $nrlanbolrec, $cdreducont, $cdtpbaixarec, $cdcartcobbol,
                                 $idcartcobrec, $idimprbolbol, $cdins1cobbol, $cdins2cobbol, $qtdiaprotbol,
                                 $cddiaprotbol, $idformcadbre, $vrpemulbrec, $vrmultabrec, $vrpemorbrec, $vrmordiabrec)
    {
        /*FunÃ§Ã£o que recebe como parÃ¢metro as chaves do boleto (nrorg, cdfilial, cdreducont) e os demais campos do
         boleto, que serÃ£o setados no banco atravÃ©s dos mÃ©todos set abaixo. */


        /** @var $boleto Boletreceber */

        /*CriaÃ§Ã£o do objeto $boleto, criado atravÃ©s da tabela BoletReceber, passando como parÃ¢metro para o mÃ©todo de
        busca as chaves da tabela*/
        $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilial, $nrlanbolrec);

        $boleto->setCdreducont($cdreducont);
        $boleto->setCdtpbaixarec($cdtpbaixarec);
        $boleto->setCdcartcobbol($cdcartcobbol);
        $boleto->setIdcartcobrec($idcartcobrec);
        $boleto->setIdimprbolbol($idimprbolbol);
        $boleto->setCdins1cobbol($cdins1cobbol);
        $boleto->setCdins2cobbol($cdins2cobbol);
        $boleto->setQtdiaprotbol($qtdiaprotbol);
        $boleto->setCddiaprotbol($cddiaprotbol);
        $boleto->setIdformcadbrec($idformcadbre);
        $boleto->setVrpemulbrec($vrpemulbrec);
        $boleto->setVrmultabrec($vrmultabrec);
        $boleto->setVrpemorbrec($vrpemorbrec);
        $boleto->setVrmordiabrec($vrmordiabrec);



        /*Caso nÃ£o de exceÃ§Ã£o executa o comitt(persist e flush), se der exceÃ§Ã£o manda a mesma para ser tratada na camada acima(FACADE) */
        try {
            $boleto->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function emiteBoleto($nrorg, $cdfilial, $nrlanbolrec)
    {
        try {
            /** @var $boleto Boletreceber */
            $boleto            = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilial, $nrlanbolrec);
            $emissaoBoleto     = EmissaoBoleto::factory($boleto);
            //VerificaÃ§Ã£o se o boleto ja foi emitido.
            if ($boleto->getIdstatbrec() != 'E'){

                // Executa validacoes quanto a emissao de boleto. Esta validacao pode variar de acordo com bancos distintos.
                $validacao         = ValidacaoEmissaoBoleto::factory($boleto);
                $validacao->valida();

                if($boleto->getReducont()->getCdbanco() != '001'){
                    $nossoNumero       = $emissaoBoleto->getNossoNumeroComDigito();
                    $codigoDeBarras    = $emissaoBoleto->getCodigoDeBarras($nossoNumero);
                    $linhaDigitavel    = $emissaoBoleto->getLinhaDigitavel($nossoNumero);
                }
                else
                {
                    $params = $emissaoBoleto->chamaRelatorio();
                    $nossoNumero       = $params["CDNOSNUMREC"];
                    $codigoDeBarras    = $params['CDBARRAREC'];
                    $linhaDigitavel    = $params['DSLINHABARREC'];
                }
                $boleto->setCdnosnumrec(substr($nossoNumero,0,-1).' '.substr($nossoNumero,-1));
                $boleto->setDslinhabarrec($linhaDigitavel);
                $boleto->setCdbarrarec($codigoDeBarras);
                $boleto->setIdstatbrec('E');

                $boleto->persist();
                \Setforms2\Shared\Registry::getEntityManager()->flush();
            }


            return $emissaoBoleto->getRelatorioBoleto($boleto->getNrorg(),$boleto->getCdreducont());
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    public function calculaValorMulta($nrlanbolrec, $cdfilial, $nrorg, $vrpermultarec)
    {
        $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilial,
            $nrlanbolrec);
        $vrfinalbrec = $boleto->getVrfinalbrec();
        return $vrfinalbrec* $vrpermultarec/100;
    }

    public function calculaPercentMulta($nrlanbolrec, $cdfilial, $nrorg, $vrmultarec)
    {
        $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilial,
            $nrlanbolrec);
        $vrfinalbrec = $boleto->getVrfinalbrec();
        return $vrmultarec/$vrfinalbrec*100;
    }

    public function calculaValorMora($nrlanbolrec,$cdfilial,$nrorg, $vrpemorbrec )
    {
        $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilial,
            $nrlanbolrec);
        $vrfinalbrec = $boleto->getVrfinalbrec();
        return $vrpemorbrec*$vrfinalbrec/100;
    }

    public  function  calculaPercentMora($nrlanbolrec,$cdfilial,$nrorg,$vrmordiabrec)
    {
        $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilial,
            $nrlanbolrec);
        $vrfinalbrec = $boleto->getVrfinalbrec();
        return $vrmordiabrec/$vrfinalbrec*100;
    }

    public function prorrogaBoletos($boletos = array(), $data, $motivo, $nrorg = null)
    {
        try{
            $nrorg          = $nrorg?:\Geral\Util\Ambiente::getNrorg();
            foreach ($boletos as $boleto)
            {
                $cdfilial    = $boleto["CDFILIAL"];
                $nrlanbolrec = $boleto["NRLANBOLREC"];

                $this->prorrogaBoleto($cdfilial,$nrlanbolrec,$nrorg,$data, $motivo);
                //tk_show_message_by_name('operacao_sucesso');

            }
        }
        catch(\Exception $e){
            throw $e;
        }

    }

    public function prorrogaBoleto($cdfilial, $nrlanbolrec, $nrorg, $dataprorrogacao, $motivo )
    {
        /** @var $boleto Boletreceber */
        try {
            //Inicia TransaÃ§Ã£o
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilial,
                $nrlanbolrec);

            //Valida a data de ProrrogaÃ§Ã£o e caso nao gere erro updata no boleto
            $this->validaProrrogacaoBoleto($dataprorrogacao, $motivo, $dataprorrogacao, $boleto);

            $dataprorrogacao = DateUtil::truncateData(DateUtil::getDataDeString($dataprorrogacao));
            $boleto->setDtvenbolrec($dataprorrogacao);

            //Caso o Boleto esteja com permissÃ£o de alteraÃ§Ã£o altera o mesmo para alterado
            if($boleto->getIdbolrecalter() == self::PERMITE_ALTERACAO_BOLETO)
                $boleto->setIdbolrecalter(self::BOLETO_ALTERADO);
            $boleto->persist();
            //Atualiza Fluxo das Contas do Boleto
            /** @var $contasreceber Contareceber*/
            //Busca as Contas que compÃµe o boleto
            $contasreceber = ContaReceber::findByNrorgAndCdfilibolrecAndNrlanbolrec($nrorg, $cdfilial,
                $nrlanbolrec);
            $controlafluxo = new FluxodeCaixa();
            foreach($contasreceber as $conta)
            {
                /** @var $conta Contareceber*/
                $dtmovfluxo = $conta->getDtmovfluxo()->format(DateUtil::FORMATO_BRASILEIRO);

                // Retira o valor do fluxo de Caixa da Data Atual da Conta
                $controlafluxo::atualizaFluxo($conta->getCdempresa(),$conta->getCdfimovflrec(),$conta->getCdclassfina(),
                    $dtmovfluxo,$conta->getVrorigrec()*-1,'E','Entrada', $nrorg);

                //Insere o valor da Conta no fluxo da data de ProrrogaÃ§Ã£o
                $controlafluxo::atualizaFluxo($conta->getCdempresa(),$conta->getCdfimovflrec(),$conta->getCdclassfina(),
                    $dataprorrogacao, $conta->getVrorigrec(),'E','Entrada', $nrorg);


                // Seta a data de prorrogaÃ§Ã£o na Conta e Executa o Commit;
                $conta->setDtmovfluxo($dataprorrogacao);
                $conta->setDtatuavenrec($dataprorrogacao);
                $conta->setCdmotipror($motivo);
                $conta->setIdmovfluxo('E');
                $conta->persist();

                //Atualiza Fluxo dos Ajustes
                $ajustes = Ajctreceber::findByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg(
                    $conta->getCdfilial(),$conta->getNrlanctonf(),$conta->getNrseqvenrec(),
                    $conta->getNrparcelarec(),$conta->getNrorg());

                /** @var  $ajuste AjctReceber*/
                foreach ($ajustes as $ajuste)
                {
                    //VeririficaÃ§Ã£o se o Ajuste Ã© de AcrÃ©scimo ou Desconto
                    /** @var  $tipoajuste Tipoajreceber*/
                    $tipoajuste = Tipoajreceber::findOneByCdajuctrecAndNrorg($ajuste->getCdajuctrec(),
                        $ajuste->getNrorg());

                    //Desconto - Acrescenta o valor do ajuste na data atual e subtrai na data de ProrrogaÃ§Ã£o
                    if($tipoajuste->getIdajuctrec() == 'D'){
                        $controlafluxo::atualizaFluxo($ajuste->getCdemmfajcntr(),$ajuste->getCdfimfajcntr(),
                            $ajuste->getCdclassfina(),$ajuste->getDtmovfluxo(),$ajuste->getVrajusterec(),'E',
                            'Entrada', $nrorg);

                        $controlafluxo::atualizaFluxo($ajuste->getCdemmfajcntr(),$ajuste->getCdfimfajcntr(),
                            $ajuste->getCdclassfina(),$dataprorrogacao,$ajuste->getVrajusterec()*-1,'E',
                            'Entrada', $nrorg);
                    }
                    //AcrÃ©scimo - Subtrai o valor do ajuste na data atual e acrescenta na data de ProrrogaÃ§Ã£o.
                    else{
                        $controlafluxo::atualizaFluxo($ajuste->getCdemmfajcntr(),$ajuste->getCdfimfajcntr(),
                            $ajuste->getCdclassfina(),$ajuste->getDtmovfluxo(),$ajuste->getVrajusterec()*-1,'E',
                            'Entrada');

                        $controlafluxo::atualizaFluxo($ajuste->getCdemmfajcntr(),$ajuste->getCdfimfajcntr(),
                            $ajuste->getCdclassfina(),$dataprorrogacao,$ajuste->getVrajusterec(),'E',
                            'Entrada', $nrorg);

                    }
                    $ajuste->setDtmovfluxo($dataprorrogacao);
                    $ajuste->persist();
                }
            }
            //Finaliza TransaÃ§Ã£o
            \Setforms2\Shared\Registry::getEntityManager()->flush();
            $connection->flush();
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }

    }

    private function validaProrrogacaoBoleto($dataprorrogacao, $motivo, $datavenc, $boleto)
    {
        /** @var $boleto Boletreceber */
        if((bool) $dataprorrogacao === false){
            ExcecaoProrrogacao::dataNaoInformada();
        }
        if((bool) $motivo === false){
            ExcecaoProrrogacao::motivoNaoInformado();
        }

        $menorData = DateUtil::getDataDeString('31/12/2037');

        $contas = Contareceber::findByNrorgAndCdfilibolrecAndNrlanbolrec(
            $boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolrec()
        );

        /** @var $conta Contareceber */
        foreach($contas as $conta){
            /** @var  $nota Notafiscal */
            $nota = Notafiscal::findOneByNrorgAndCdfilialAndNrlanctonf(
                $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf()
            );
            if(DateUtil::comparaDatas($nota->getDtemissao(), $menorData)){
                $menorData = $nota->getDtemissao();
            }
        }

        if(DateUtil::comparaDatas(DateUtil::getDataDeString($datavenc), $menorData)){
            throw new \Exception('Data de prorrogação deve ser maior que a data de emissão da nota fiscal.');
        }

    }

    public static function CancelaBoleto()
    {
        try{
            /** @var $boleto Boletreceber */
            $listaBoletos =  tk_get_env_var("boletos_marcados");
            foreach($listaBoletos as $boleto){

                //Monta o objeto $boleto
                $nrorg         = $boleto['NRORG'];
                $cdfilial      = $boleto['CDFILIAL'];
                $nrlanbolrec  = $boleto['NRLANBOLREC'];
                $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilial, $nrlanbolrec);

                //Validacao CNAB
                if ($boleto->getIdstatbrec() == 'E'){

                    if (self::VerificaSituacaoCnab($boleto->getIdsitucnabr()) == true){
                        $mensagem = ("Existem boletos marcados enviados via CNAB. Ao confirmar a operaÃ§Ã£o os mesmos serÃ£o
                             excluÃ­dos da remessa. Confirma o Cancelamento?");
                        $confirma = array(
                            "label"=> "Sim",
                            "method"=> array("\\Financeiro\\Logic\\ContasReceber\\Boleto\\OperacoesBoleto", "setCancelamento"),
                            'params_arr' => array($boleto),
                        );
                        $cancela = array(
                            "label"=> "NÃ£o",
                            "method"=> array(),
                            'params_arr' => array(),
                        );
                        tk_confirm($mensagem, $confirma, $cancela);
                    }
                    else{
                        self::setCancelamento($boleto);
                    }

                }
                else{
                    //Se o boleto nao estiver emitido e estiver associado separa os mesmos.
                    $contasreceber = ContaReceber::findByNrorgAndCdfilibolrecAndNrlanbolrec($boleto->getNrorg(),
                        $boleto->getCdfilial(),
                        $boleto->getNrlanbolrec());
                    if(count($contasreceber)>1){

                        //Verififca se o boleto estÃ¡ compensado
                        if (self::verificaBolCompensado($boleto) == true)
                            ExcecaoCancelamento::sepBolComp();

                        self::separaBoletos($boleto);
                    }
                }
            }

            tk_reload_data_with_filter();
        }
        catch(\Exception $e){
            \Geral\Util\TratamentoDeMensagens::exibirMensagens($e);
        }
    }

    private function separaBoletos(\Financeiro\Entities\Boletreceber $boleto){
        try{


            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            self::cancelaRemessa($boleto);

            if((int)$boleto->getNrparcbolrec() > 0)
                ExcecaoCancelamento::opBloqBolParc();

            $rep = \Financeiro\Entities\Boletreceber::getRepository();

            $duplicatas = $rep-> getBoletoDup($boleto->getNrorg(),$boleto->getCdfilial(),$boleto->getNrlanbolrec());

            if((count($duplicatas) == 1) and($duplicatas[0]['CDFILIALDUP'] != null) and($duplicatas[0]
                    ['NRLANCTODUP'] != null)){
                ExcecaoCancelamento::cntDupNSep();
            }
            $qtboletos = count($duplicatas);

            $prxboleto =  $novocodigo = NovoCodigo::geraCodigo($boleto->getNrorg(),self::PREFIXO_NOVOCODIGO_BOLREC.
                \Geral\Util\Ambiente::getCdfilial(),
                $qtboletos,self::TAMANHO_NRLANBOLREC);

            // Gera novo boleto
            $contasreceber = Contareceber::findByCdfilibolrecAndNrlanbolrecAndNrorg($boleto->getCdfilial(),
                $boleto->getNrlanbolrec(),
                $boleto->getNrorg());

            $cdfilialdup = 'XXXX';
            $nrlancdup = 'XXXXXXXXXX';
            $cdfilial = 'XXX';
            $nrlanctonf = 'XXXXXXXXXX';
            $vrboleto = 0;
            $vrajboleto = 0;

            $qtcontas = 0;

            foreach($contasreceber as $conta){
                // Grava os boletos que estÃ£o ligados a uma duplicata
                $qtcontas++;
                if( ($conta->getCdfilialdup() != null) and $conta->getNrlanctodup() ){
                    if($conta->getCdfilialdup() != $cdfilialdup or $conta->getNrlanctodup() != $nrlancdup){

                        $sqlvrboletodup = $rep->getValorBoletoDup($conta->getNrorg(),
                            $conta->getCdfildialdup(),
                            $conta->getNrlanctodup());
                        $vrboleto = $sqlvrboletodup[0]['VRATUAREC'];

                        $sqlajboldup = $rep->getAjBolDup($conta->getNrorg(),
                            $conta->getCdfildialdup(),
                            $conta->getNrlanctodup());
                        $vrajboleto = $sqlajboldup[0]['VRAJDUPBOLET'];

                        $novoboleto = new Boletreceber();
                        $novoboleto->setCdfilial(\Geral\Util\Ambiente::getCdfilial());
                        $novoboleto->setNrlanbolrec($prxboleto);
                        $novoboleto->setCdcliente($boleto->getCdcliente());
                        $novoboleto->setCdEmpresa($boleto->getCdempresa());
                        $novoboleto->setVrbolrec($boleto->getVrbolrec());
                        $novoboleto->setVrpermulbrec($boleto->getVrpemulbrec());
                        $novoboleto->setVrmultabrec($boleto->getVrmultabrec());
                        $novoboleto->setVrpemorbrec($boleto->getVrpemorbrec());
                        $novoboleto->setVrmordiabrec($boleto->getVrmordiabrec());
                        $novoboleto->setVrfinalbrec(round($vrboleto, 2) + round($vrajboleto, 2));
                        $novoboleto->setDtvenbolrec($conta->getDtorigvenrec());
                        $novoboleto->setCdtpbaixarec(null);
                        $novoboleto->setIdstatbrec(self::STATUS_BOLETO_PENDENTE);
                        $novoboleto->setCdnosnumrec('');
                        $novoboleto->setDtinclusao(DateUtil::getDataAtual());
                        $novoboleto->persist();

                        $boletoatual = $prxboleto;

                        // -- Buscas os ajustes antigos para jogar nos novos boletos.
                        $sqlajboldupaux = $rep->getAjbolDupAux($conta->getCdfilialdup(),
                            $conta->getNrlanctodup(),
                            $boleto->getCdfilial(),
                            $boleto->getNrlanbolrec(),
                            \Geral\Util\Ambiente::getNrorg());

                        foreach ($sqlajboldupaux as $ajuste){
                            $novoajuste = new Ajbolrec();
                            $novoajuste->setCdfilial(\Geral\Util\Ambiente::getCdfilial());
                            $novoajuste->setNrlanbolrec($prxboleto);
                            $novoajuste->setCdajuctrec($ajuste['CDAJUCTREC']);
                            $novoajuste->setVrajbolrec($ajuste['VRAJDUPBOLET']);
                            $novoajuste->setDtinclusao(DateUtil::getDataAtual());
                            $novoajuste->persist();

                        }
                        $prxboleto = (str_pad($prxboleto+$qtcontas,10,'0',STR_PAD_LEFT));

                        //Atualiza fluxo de caixa da conta a pagar
                        FluxodeCaixa::atualizaFluxo($conta->getCdempresa(),
                            $conta->getCdfimovflrec(),
                            $conta->getCdclassfina(),
                            $conta->getDtmovfluxo(),
                            $conta->getVrorigrec()* -1,
                            $conta->getIdmovfluxo(),
                            'Entrada', $nrorg);


                        FluxodeCaixa::atualizaFluxo($conta->getCdempresa(),
                            $conta->getCdfimovflrec(),
                            $conta->getCdclassfina(),
                            $conta->getDtorigvenrec(),
                            $conta->getVrorigrec(),
                            $conta->getIdmovfluxo(),
                            'Entrada', $nrorg);

                    }
                    // updata a conta a receber com o numero e filiproc do novo boleto
                    $conta->setCdfiliproc(\Geral\Util\Ambiente::getCdfilial());
                    $conta->setNrlanbolrec($boletoatual);
                    $conta->persist();

                }
                else{
                    if( ($cdfilial != $conta->getCdfilial()) or ($nrlanctonf != $conta->getNrlanctonf()) ){

                        //Verifica a quantidade de contas a serem atualizadas pelo procedimento  GERA_BOLETOS
                        $contaatu = Contareceber::findByCdfilialAndNrlanctonfAndCdfilibolrecAndNrlanbolrecAndNrlanctodupAndNrorg(
                            $conta->getCdfilial(),
                            $conta->getNrlanctonf(),
                            $conta->getCdfilibolrec(),
                            $conta->getNrlanbolrec(),
                            null,
                            $conta->getNrorg());
                        $qtcontas = count($contaatu);
                        self::execGeraBoletos($conta,$prxboleto);

                        $cdfilial = $conta->getCdfilial();
                        $nrlanctonf = $conta->getNrlanctonf();
                        $prxboleto = (str_pad($prxboleto+$qtcontas,10,'0',STR_PAD_LEFT));
                    }

                }
            }
            //Apaga Ajuste
            $delajustes =  Ajbolrec::findOneByCdfilialAndNrlanbolrecAndNrorg
            ($boleto->getCdfilial(),$boleto->getNrlanbolrec(),
                $boleto->getNrorg());
            if((bool)$delajustes === true){
                foreach($delajustes as $ajuste){
                    $ajuste->delete();
                }
            }

            //Apaga Boleto Agrupado
            $boleto->delete();


            \Setforms2\Shared\Registry::getEntityManager()->flush();
            $connection->flush();
            $connection->commit();

        }
        catch(\Exception $e){
            $connection->rollback();
            throw $e;
        }

    }

    private function execGeraBoletos(\Financeiro\Entities\Contareceber $conta, $prxboleto){

        try{

            $procedure = new Procedure('GERA_BOLETOS');
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $conta->getCdfilial(),Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT,$conta->getNrlanctonf(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILILANC', Param::PARAM_INPUT, \Geral\Util\Ambiente::getCdfilial(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANBOLREC', Param::PARAM_INPUT,$prxboleto, Param::PARAM_TYPE_STR));

            $procedure->addParam(new Param('P_CDTPCTRECEB', Param::PARAM_INPUT, $conta->getCdtpctreceb(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDEXCBOLREC', Param::PARAM_INPUT, 'S', Param::PARAM_TYPE_STR));

            $procedure->addParam(new Param('P_CDFILBOLORI', Param::PARAM_INPUT, $conta->getCdfilibolrec(), Param::PARAM_TYPE_STR));

            $procedure->addParam(new Param('P_NRLANBOLORI', Param::PARAM_INPUT, $conta->getNrlanbolrec(), Param::PARAM_TYPE_STR));

            $procedure->call();
        }
        catch(\Exception $e){
            throw $e;

        }


    }

    private static function verificaBolCompensado(\Financeiro\Entities\Boletreceber $boleto)
    {
        try{
            //Verifica se o boleto esta compensado
            $ajbolcomp = Ajbolrec::findByCdfilialAndNrlanbolrecAndIdorigajbrecAndNrorg(
                $boleto->getCdfilial(),
                $boleto->getNrlanbolrec(),
                self::ID_AJ_BOL_COMP,
                $boleto->getNrorg());
            return((bool)$ajbolcomp);
        }
        catch(\Exception $e){
            throw $e;
        }



    }

    private function setCancelamento(\Financeiro\Entities\Boletreceber $boleto){
        try{
            //Cancela a emissÃ£o do boleto
            $boleto->setIdstatbrec('A');
            $boleto->setIdsitucnabr(self::ID_CNAB_N_ENVIADO);
            $boleto->setNrcnabrecrem(null);
            $boleto->setDslinhabarrec(null);
            $boleto->setCdbarrarec(null);
            $boleto->setCdnosnumrec('');
            $boleto->setCddepidenrec(null);
            $boleto->setIdbolemitido('');

            $boleto->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

        }catch(\Exception $e){
            throw $e;
        }
    }

    public function InsereAjuste($nrorg,$cdfilial,$nrlanbolrec,$vrajuctrec,$idajuctrec)
    {
        try{

            /** @var $boleto Boletreceber */
            $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg,$cdfilial,$nrlanbolrec);

            //updata o valor final e apaga os dados de emissÃ£o do boleto
            $valorAnterior = $boleto->getVrfinalbrec();
            $boleto->setVrfinalbrec(round($valorAnterior + self::CalcValorAjuste($vrajuctrec,$idajuctrec), 2));
            $boleto->setIdstatbrec('A');
            $boleto->setDslinhabarrec(null);
            $boleto->setCdbarrarec(null);
            $boleto->setCdnosnumrec('');
            $boleto->setIdbolrecalter(1);

            $boleto->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

        }
        catch(\Exception $e){
            throw $e;
        }

    }

    public function ExcluiAjuste($nrorg,$cdfilial,$nrlanbolrec,$vrajuctrec,$idajuctrec)
    {
        try{

            /** @var $boleto Boletreceber */
            $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg,$cdfilial,$nrlanbolrec);

            //updata o valor final e apaga os dados de emissÃ£o do boleto
            $valorAnterior = $boleto->getVrfinalbrec();
            $boleto->setVrfinalbrec(round($valorAnterior - self::CalcValorAjuste($vrajuctrec,$idajuctrec), 2));
            $boleto->setIdstatbrec('A');
            $boleto->setDslinhabarrec(null);
            $boleto->setCdbarrarec(null);
            $boleto->setCdnosnumrec('');
            $boleto->setIdbolrecalter(1);


            $boleto->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();
        }
        catch(\Exception $e){
            throw $e;
        }
    }

    private function CalcValorAjuste($vrajuctrec,$idajuctrec)
    {
        if ($idajuctrec == 'D'){//Desconto
            return round(-$vrajuctrec, 2);

        } else{//Acrescimo
            return round($vrajuctrec, 2);
        }
    }

    public static function VerificaSituacaoCnab($idsitcnab)
    {
        $exibirConfirm = false;
        if (((bool) $idsitcnab === true ) and !(in_array($idsitcnab,array('00','01')))){
            /** @var $geralrec GeralRec */
            $geralrec = GeralRec::findOneByNrorg(\Geral\Util\Ambiente::getNrorg());
            if ($geralrec->getIdutctrlcanremr() == 'S'){
                ExcecaoCnab::Cnab_N_Alterado();
            }
            else{
                $exibirConfirm = true;
            }
        }
        return $exibirConfirm;

    }
    public static function ValidaBolEmitido(\Financeiro\Entities\Boletreceber $boleto){

        if($boleto->getIdstatbrec() == 'E'){
            $mensagem = ("Boleto jÃ¡ foi emitido. A confirmaÃ§Ã£o desta operaÃ§Ã£o farÃ¡ com que a emissÃ£o seja cancelada.
                          Deseja continuar?");
            $confirma = array(
                "label"=> "Sim",
                "method"=> function(){},
                'params_arr' => array(),
            );
            $cancela = array(
                "label"=> "NÃ£o",
                "method"=> array(),
                'params_arr' => array(),
            );
            tk_confirm($mensagem, $confirma, $cancela);
        }

    }



    public static function GeraBoleto($nrorg, $filial, $boletos, $datavencimento, $cdreducont, $cdtpbaixarec)
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        try {

            $datavencimento = DateUtil::truncateData(DateUtil::getDataDeString($datavencimento));
            $novocodigo = NovoCodigo::geraCodigo($nrorg, self::PREFIXO_NOVOCODIGO_BOLREC . $filial,
                1, self::TAMANHO_NRLANBOLREC);
            //Busca valor total dos boletos
            $valortotal = 0;
            foreach ($boletos as $boleto) {
                $valortotal += $boleto['VRFINALBREC'];
            }
            /** @var $boleto BoletReceber */
            //Utilizado para buscar dados do boleto que serao setados no novo boleto
            $boleto =  Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $boletos[0]['CDFILIAL'], $boletos[0]['NRLANBOLREC']);

            $boletonovo = new Boletreceber();
            $boletonovo->setNrorg($nrorg);
            $boletonovo->setNrlanbolrec($novocodigo);
            $boletonovo->setCdfilial($filial);
            $boletonovo->setVrbolrec($valortotal);
            $boletonovo->setVrfinalbrec($valortotal);
            $boletonovo->setDtvenbolrec($datavencimento);
            $boletonovo->setCdreducont($cdreducont);
            $boletonovo->setCdtpbaixarec($cdtpbaixarec);
            $boletonovo->setCdcliente($boleto->getCdcliente());
            $boletonovo->setNrlicencauso($boleto->getNrlicencauso());
            $boletonovo->setCdempresa($boleto->getCdempresa());
            $boletonovo->setIdprotestrec($boleto->getIdprotestrec());
            $boletonovo->setIdbolrecalter(self::IDBOLRECALTER_DEFAULT);
            $boletonovo->setIdformcadbrec(self::IDFORMCADBREC_DEFAULT);
            $boletonovo->setIdstatbrec(self::STATUS_BOLETO_PENDENTE);
            $boletonovo->setIdsitucnabr(self::ID_CNAB_N_ENVIADO);
            $boletonovo->setNrparcbolrec('00');
            $boletonovo->setDtinclusao(DateUtil::getDataAtual());
            $boletonovo->setDtultatu(DateUtil::getDataAtual());
            $boletonovo->setIdativo('S');

            $boletonovo->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush(); // Sem esse comando gera parent key not found
            // na inserção do ajuste.


            foreach ($boletos as $boleto){

                //Busca as Contas que compõe o boleto
                $contasreceber = ContaReceber::findByNrorgAndCdfilibolrecAndNrlanbolrec($nrorg, $boleto['CDFILIAL'],
                    $boleto['NRLANBOLREC']);

                /** @var $conta ContaReceber*/
                foreach ($contasreceber as $conta){
                    if((DateUtil::comparaDatas($conta->getDtorigvenrec(),$datavencimento, 'IGUAL') == false)
                        or (DateUtil::comparaDatas($conta->getDtatuavenrec(),$datavencimento, 'IGUAL') == false))
                    {
                        //Subtrai o valor do fluxo da conta na data anterior
                        FluxodeCaixa::atualizaFluxo($conta->getCdempresa(),$conta->getCdfimovflrec(),$conta->getCdclassfina(),
                            $conta->getDtmovfluxo(),$conta->getVrorigrec()*-1,$conta->getIdmovfluxo(),
                            'Entrada', $nrorg);
                        //Acresce o valor do fluxo da conta na nova data de vencimento
                        FluxodeCaixa::atualizaFluxo($conta->getCdempresa(),$conta->getCdfimovflrec(),$conta->getCdclassfina(),
                            $datavencimento,$conta->getVrorigrec(),$conta->getIdmovfluxo(),
                            'Entrada', $nrorg);
                        $conta->setNrlanbolrec($novocodigo);
                        $conta->setCdfilibolrec($filial);
                        $conta->setDtatuavenrec($datavencimento);
                        $conta->setDtmovfluxo($datavencimento);
                        $conta->setCdmotipror(null);
                        $conta->persist();

                        $ajustes = Ajctreceber::findByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg(
                            $conta->getCdfilial(),$conta->getNrlanctonf(),$conta->getNrseqvenrec(),
                            $conta->getNrparcelarec(),$conta->getNrorg());
                        /** @var  $ajuste AjctReceber*/
                        foreach ($ajustes as $ajuste)
                        {
                            //VeririficaÃ§Ã£o se o Ajuste Ã© de AcrÃ©scimo ou Desconto
                            /** @var  $tipoajuste Tipoajreceber*/
                            $tipoajuste = Tipoajreceber::findOneByCdajuctrecAndNrorg($ajuste->getCdajuctrec(),
                                $ajuste->getNrorg());


                            if($tipoajuste->getIdajuctrec() == 'D'){
                                FluxodeCaixa::atualizaFluxo($ajuste->getCdemmfajcntr(),$ajuste->getCdfimfajcntr(),
                                    $ajuste->getCdclassfina(),$ajuste->getDtmovfluxo(),$ajuste->getVrajusterec(),'E',
                                    'Entrada', $nrorg);
                            }
                            else{

                                FluxodeCaixa::atualizaFluxo($ajuste->getCdemmfajcntr(),$ajuste->getCdfimfajcntr(),
                                    $ajuste->getCdclassfina(),$ajuste->getDtmovfluxo(),$ajuste->getVrajusterec()*-1,'E',
                                    'Entrada', $nrorg);
                            }
                            if($tipoajuste->getIdajuctrec() == 'D'){
                                FluxodeCaixa::atualizaFluxo($ajuste->getCdemmfajcntr(),$ajuste->getCdfimfajcntr(),
                                    $ajuste->getCdclassfina(),$datavencimento,$ajuste->getVrajusterec()*-1,'E',
                                    'Entrada', $nrorg);
                            }
                            else{
                                FluxodeCaixa::atualizaFluxo($ajuste->getCdemmfajcntr(),$ajuste->getCdfimfajcntr(),
                                    $ajuste->getCdclassfina(),$datavencimento,$ajuste->getVrajusterec(),'E',
                                    'Entrada', $nrorg);
                            }
                            $ajuste->setDtmovfluxo($datavencimento);
                            $ajuste->persist();
                        }
                    }
                    else{
                        $conta->setCdfilibolrec($filial);
                        $conta->setNrlanbolrec($novocodigo);
                        $conta->persist();
                    }

                }
                //TODO VERIFICAR ADIANTAMENTO

                // -- Grava os ajustes no novo boleto
                $ajustes = Ajbolrec::findByCdfilialAndNrlanbolrecAndNrorg($boleto['CDFILIAL'],$boleto['NRLANBOLREC'],$boleto['NRORG']);

                /** @var $ajuste Ajbolrec */
                foreach ($ajustes as $ajuste){
                    // -- Verifica se o ajuste jÃ¡ foi gravado no novo boleto
                    /** @var $ajusteaux Ajbolrec */
                    $ajusteaux = Ajbolrec::findOneByCdfilialAndNrlanbolrecAndCdajuctrecAndNrorg(
                        $filial,$novocodigo,$ajuste->getCdajuctrec(),$boleto['NRORG']);
                    if ((bool) $ajusteaux === false){
                        $novoajuste = new Ajbolrec();
                        $novoajuste->setCdfilial($filial);
                        $novoajuste->setNrlanbolrec($novocodigo);
                        $novoajuste->setCdajuctrec($ajuste->getCdajuctrec());
                        $novoajuste->setVrajbolrec($ajuste->getVrajbolrec());
                        $novoajuste->setNrorg($ajuste->getNrorg());
                        $novoajuste->setDtinclusao(DateUtil::getDataAtual());
                        $novoajuste->persist();
                        \Setforms2\Shared\Registry::getEntityManager()->flush();// Sem esse comando n vai entrar no else
                        //quando necessÃ¡rio
                    } else{
                        $ajusteaux->setVrajbolrec($ajusteaux->getVrajbolrec()+$ajuste->getVrajbolrec());
                        $ajusteaux->persist();
                    }

                }
                // Apaga os boletos associados
                Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec(
                    $nrorg,$boleto['CDFILIAL'],$boleto['NRLANBOLREC'])->delete();


            }
            \Setforms2\Shared\Registry::getEntityManager()->flush();
            $connection->flush();
            $connection->commit();


        }catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public static function verificaAdiantamento($cdempresa,$cdcliente,$nrorg)
    {
        /** @var $adiantarecbeRepository \Financeiro\Repositories\Adiantareceb*/
        $adiantarecbeRepository = \Financeiro\Entities\Adiantareceb::getRepository();
        $adiantamentos = $adiantarecbeRepository->verificaAdiantamento($cdempresa,$cdcliente,$nrorg);
        return $adiantamentos;

    }
    public static function confirmaAdiantamento(Boletreceber $boleto, $valoradiantamento, $codigoajuste, $adiantamentos)
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();
            /** @@var $boleto Boletreceber */
            // Se possuir nÃºmero de remessa cancela a mesma
            if ((bool)$boleto->getNrcnabrecrem() === true)
                self::cancelaRemessa($boleto);

            //Insere Ajuste
            $ajuste = new Ajbolrec();
            $ajuste->setCdajuctrec($codigoajuste);
            $ajuste->setVrajbolrec($valoradiantamento);
            $ajuste->setCdfilial($boleto->getCdfilial());
            $ajuste->setNrlanbolrec($boleto->getNrlanbolrec());
            $ajuste->setNrorg($boleto->getNrorg());
            $ajuste->setDtinclusao(DateUtil::getDataAtual());
            $ajuste->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

            // Insere Adiantamento
            foreach ($adiantamentos as $item){
                $adiantareceb = new \Financeiro\Entities\Adiantareceb();
                $novocodigo = NovoCodigo::geraCodigo(Ambiente::getNrorg(), 'ADIANTARECEB');
                $adiantareceb->setNradiantareceb($novocodigo);
                $adiantareceb->setCdfiliajbolr($boleto->getCdfilial());
                $adiantareceb->setNrlanbolrec($boleto->getNrlanbolrec());
                $adiantareceb->setCdajuctrec($codigoajuste);
                $adiantareceb->setCdfilial($item['CDFILIAL']);
                $adiantareceb->setNrlanctonf($item['NRLANCTONF']);
                $adiantareceb->setNrseqvenrec($item['NRSEQVENREC']);
                $adiantareceb->setNrparcelarec($item['NRPARCELAREC']);
                $adiantareceb->setVradiantarec($item['VRDESCONTO']);
                $adiantareceb->setNrorg($item['NRORG']);
                $adiantareceb->setDtinclusao(DateUtil::getDataAtual());
                $adiantareceb->persist();

            }

            $boleto->setVrfinalbrec($boleto->getVrfinalbrec() - $valoradiantamento);
            $boleto->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;

        }

    }
    public static function verificaDevolucao($cdempresa,$cdcliente,$nrorg)
    {
        /** @var $devolurecbeRepository \Financeiro\Repositories\Adiantareceb*/
        $devolurecbeRepository = \Financeiro\Entities\Adiantareceb::getRepository();
        $devolucoes = $devolurecbeRepository->verificaDevolucao($cdempresa,$cdcliente,$nrorg);
        return $devolucoes;

    }
    public static function confirmaDevolucao(Boletreceber $boleto, $valordevolucao, $codigoajuste, $devolucoes)
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();
            /** @@var $boleto Boletreceber */
            // Se possuir nÃºmero de remessa cancela a mesma
            if ((bool)$boleto->getNrcnabrecrem() === true)
                self::cancelaRemessa($boleto);

            //Insere Ajuste
            $ajuste = new Ajbolrec();
            $ajuste->setCdajuctrec($codigoajuste);
            $ajuste->setVrajbolrec($valordevolucao);
            $ajuste->setCdfilial($boleto->getCdfilial());
            $ajuste->setNrlanbolrec($boleto->getNrlanbolrec());
            $ajuste->setNrorg($boleto->getNrorg());
            $ajuste->setDtinclusao(DateUtil::getDataAtual());
            $ajuste->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

            // Insere Adiantamento
            foreach ($devolucoes as $item){
                $devolucaorec = new \Indeterminado\Entities\Devolucaorec();
                $devolucaorec->setCdfilibolrec($boleto->getCdfilial());
                $devolucaorec->setNrlanbolrec($boleto->getNrlanbolrec());
                $devolucaorec->setCdajuctrec($codigoajuste);
                $devolucaorec->setCdfilial($item['FILIDEVO']);
                $devolucaorec->setNrlanctonf($item['NRLANCTONF']);
                $devolucaorec->setVrdevolucaorec($item['VRDESCONTO']);
                $devolucaorec->setNrorg($item['NRORG']);
                $devolucaorec->setDtinclusao(DateUtil::getDataAtual());
                $devolucaorec->persist();

            }
            $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($item['FILIDEVO'], $item['NRLANCTONF'], $item['NRORG']);
            $notafiscal->setVrdevolnfutl($notafiscal->getVrdevolnfutl() + $item['VRDESCONTO']);
            $notafiscal->setNrnotadevol($item['VRDESCONTO']);
            $notafiscal->persist();

            $boleto->setVrfinalbrec($boleto->getVrfinalbrec() - $valordevolucao);
            $boleto->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;

        }

    }

    public static function cancelaRemessa(Boletreceber $boleto){
        try{

            //Se o boleto estiver ligado a uma remessa
            if ((bool)$boleto->getNrcnabrecrem() === true){
                //Apaga o vÃ­nculo do boleto com a remessa e coloca o Status de "NÃ£o enviado"
                $boleto->setIdsitucnabr(self::ID_CNAB_N_ENVIADO);
                $boleto->persist();
                /** @var $cnab Cnabrec  */
                //Atualiza o valor e a quantidade de boletos da remessa ao qual o boleto pertencia
                $cnab = Cnabrec::findOneByCdempresaAndNrcnabrecAndIdtipcnabrecAndNrorg($boleto->getCdempresa(),
                    $boleto->getNrcnabrecrem(),
                    '0',
                    $boleto->getNrorg());
                $cnab->setVrtotbolcnabrec(round($cnab->getVrtotbolcnabrec() - $boleto->getVrbolrec(), 2));
                $cnab->setQtbolenvcnabrec($cnab->getQtbolenvcnabrec() -1);
                $cnab->persist();
            }

        } catch (\Exception $e) {
            throw $e;
        }

    }

    public static function ParcelaBoleto($parcelas = array(),$cdfilial,$nrlanbolrec,$nrorg,$nrultparc){

        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            $nrlanbolrecparc = '';
            $cdfilialparc = '';
            $novasparcelas = array();

            foreach ($parcelas as $parcela) {
                //Valida somente parcelas alteradas ou incluÃ­das
                if ($parcela['NRPARCBOLREC'] >= $nrultparc) {
                    if (($parcela['NRPARCBOLREC'] == $nrultparc) or ($nrultparc == '00')) {
                        $nrlanbolrecparc = $parcela['NRLANBOLREC'];
                        $cdfilialparc = $parcela['CDFILIAL'];
                        if ($parcela['IDSITUCNABR'] == '10' or $parcela['IDSITUCNABR'] == '20') {
                            $boleto = Boletreceber::findOneByCdfilialAndNrlanbolrecAndNrorg($parcela['CDFILIAL'],
                                $parcela['NRLANBOLREC'],
                                $parcela['NRORG']);
                            self::cancelaRemessa($boleto);
                        }
                    }
                    $novasparcelas[] = array(
                        'VRFINALBREC' =>$parcela['VRFINALBREC'],
                        'DTVENBOLREC' =>$parcela['DTVENBOLREC'],
                        'CDMOTIPARC'  =>$parcela['CDMOTIPARC']
                    );
                }
            }
            /** @var $tprecboleto Tprecboleto */
            $tprecboleto = Tprecboleto::findOneByCdfilialAndNrlanbolrecAndNrorg($cdfilial,
                $nrlanbolrec,
                $nrorg);
            if ((bool) $tprecboleto === true){
                $tprecboleto->delete();
                $tprecboleto->persist();
            }
            self::SetParcelamento($nrorg, $cdfilialparc, $nrlanbolrecparc, $novasparcelas);

            \Setforms2\Shared\Registry::getEntityManager()->flush();
            $connection->flush();
            $connection->commit();
        }catch (\Exception $e) {
            throw $e;
            $connection->rollback();
        }
    }
    private static function SetParcelamento($nrorg, $cdfilial, $nrlanbolrec, $parcelas = array()){
        /** @var $rep \Financeiro\Repositories\Boletreceber */
        $rep = \Financeiro\Entities\Boletreceber::getRepository();
        $novocodigo = NovoCodigo::geraCodigo($nrorg,
            self::PREFIXO_NOVOCODIGO_BOLREC.$cdfilial,
            count($parcelas),self::TAMANHO_NRLANBOLREC);

        //Busca o boleto original
        /** @var $boletooriginal Boletreceber */
        $boletooriginal = Boletreceber::findOneByCdfilialAndNrlanbolrecAndNrorg($cdfilial,
            $nrlanbolrec,
            $nrorg);
        $vrantboleto = $boletooriginal->getVrfinalbrec();
        //NÃºmero do parcelamento original
        if((bool)$boletooriginal->getNrorigbolrec() === false)
            $nrorigbolrec = $nrlanbolrec;
        else
            $nrorigbolrec = $boletooriginal->getNrorigbolrec();

        //Busca as Contas do Boleto Parcelado
        $contasoriginais = $rep->getContaOriginalParcelamento($nrorg, $nrlanbolrec, $cdfilial);

        //Busca maior nÃºmero das parcelas do boleto
        $sqlmaxparcela= $rep->getMaxParcela($nrorg,$nrlanbolrec,$cdfilial,$nrorigbolrec);

        $maiornrparcela = $sqlmaxparcela[0]['ULTPARC'];

        if((int)$maiornrparcela == 0)
            $maiornrparcela = 1;

        $incboletos = 0;
        $nrnovoboleto = '';

        for($i = 0;$i<count($parcelas);$i++){

            //Veirifica se a data Ã© do tipo Date se nao for converte pra Date a Trunca a mesma.
            $dtvenbolrec = $parcelas[$i]['DTVENBOLREC'];
            if(strlen($dtvenbolrec)>10)
                $dtvenbolrec = DateUtil::truncateString($dtvenbolrec);
            if($dtvenbolrec instanceof \Setforms2\Shared\ORM\DateTime === false)
                $dtvenbolrec = DateUtil::getDataDeString($dtvenbolrec);
            $dtvenbolrec = DateUtil::truncateData($dtvenbolrec);

            if($i == 0){//Atualiza boleto original
                $boletooriginal->setVrfinalbrec($parcelas[$i]['VRFINALBREC']);
                $boletooriginal->setDtvenbolrec($dtvenbolrec);
                $boletooriginal->setDslinhabarrec(null);
                $boletooriginal->setCdbarrarec(null);
                $boletooriginal->setCdnosnumrec(null);

                if((bool) $boletooriginal->getIdimprbolbol() === false){
                    $boletooriginal->setIdimprbolbol('B');
                }

                $boletooriginal->setTxmensbolbol(null);
                $boletooriginal->setIdsitucnabr(self::ID_CNAB_N_ENVIADO);
                $boletooriginal->setNrcnabrecrem(null);
                $boletooriginal->setVrbolrec(round($parcelas[$i]['VRFINALBREC'], 2));
                $boletooriginal->setIdstatbrec(self::STATUS_BOLETO_PENDENTE);
                $boletooriginal->setCdmotiparc($parcelas[$i]['CDMOTIPARC']);
                $boletooriginal->setNrbaixarec(null);
                $boletooriginal->setCdfilibairec(null);
                $boletooriginal->setNrparcbolrec(str_pad($maiornrparcela,2,'0',STR_PAD_LEFT));
                $boletooriginal->setNrorigbolrec($nrorigbolrec);

                if($boletooriginal->getIdformcadbrec() === false){
                    $boletooriginal->setIdformcadbrec('1');
                }
                $boletooriginal->persist();
            }
            else{

                //Insere as novas parcelas geradas no parcelamento
                $incboletos++;
                $boletonovo = new Boletreceber();
                $boletonovo->setNrorg($nrorg);
                $boletonovo->setCdfilial($boletooriginal->getCdfilial());
                $boletonovo->setNrlanbolrec($novocodigo);
                $boletonovo->setCdcliente($boletooriginal->getCdcliente());
                $boletonovo->setCdempresa($boletooriginal->getCdempresa());
                $boletonovo->setDtvenbolrec($dtvenbolrec);
                $boletonovo->setVrpemulbrec($boletooriginal->getVrpemulbrec());
                $boletonovo->setVrmultabrec($boletooriginal->getVrmultabrec());
                $boletonovo->setVrpemorbrec($boletooriginal->getVrpemorbrec());
                $boletonovo->setVrmordiabrec($boletooriginal->getVrmordiabrec());
                $boletonovo->setCdtpbaixarec($boletooriginal->getCdtpbaixarec());
                $boletonovo->setCdreducont($boletooriginal->getCdreducont());
                $boletonovo->setCdcartcobbol($boletooriginal->getCdcartcobbol());
                $boletonovo->setNrorigbolrec($nrorigbolrec);
                $boletonovo->setIdcartcobrec($boletooriginal->getIdcartcobrec());
                $boletonovo->setCddiaprotbol($boletooriginal->getCddiaprotbol());
                $boletonovo->setQtdiaprotbol($boletooriginal->getQtdiaprotbol());
                $boletonovo->setCdins1cobbol($boletooriginal->getCdins1cobbol());
                $boletonovo->setCdins2cobbol($boletooriginal->getCdins2cobbol());
                $boletonovo->setCdfilibairec(null);
                $boletonovo->setNrbaixarec(null);
                $boletonovo->setVrbolrec($parcelas[$i]['VRFINALBREC']);
                $boletonovo->setVrfinalbrec($parcelas[$i]['VRFINALBREC']);
                $boletonovo->setIdstatbrec(self::STATUS_BOLETO_PENDENTE);
                $boletonovo->setDslinhabarrec(null);
                $boletonovo->setCdbarrarec(null);
                $boletonovo->setCdnosnumrec(null);
                $boletonovo->setCdmotiparc($parcelas[$i]['CDMOTIPARC']);
                $boletonovo->setNrparcbolrec(str_pad($maiornrparcela+$incboletos,2,'0',STR_PAD_LEFT));
                $boletonovo->setIdimprbolbol($boletooriginal->getIdimprbolbol());
                $boletonovo->setIdformcadbrec($boletooriginal->getIdformcadbrec());
                $boletonovo->setIdbolrecalter('1');
                $boletonovo->setIdsitucnabr(self::ID_CNAB_N_ENVIADO);
                $boletonovo->setDtinclusao(DateUtil::getDataAtual());

                $boletonovo->persist();
                \Setforms2\Shared\Registry::getEntityManager()->flush();

                $nrnovoboleto = $novocodigo;
                $novocodigo = (str_pad($novocodigo+1,10,'0',STR_PAD_LEFT));

            }
            $vrtotconta = 0;
            $contadorcontas = 0;
            foreach ($contasoriginais as $conta){
                $contadorcontas++;
                $sqlmaxconta = $rep->getMaxCntParcReceber(
                    $conta['NRORG'],
                    $conta['CDFILIAL'],
                    $conta['NRLANCTONF'],
                    $conta['NRSEQVENREC']
                );
                $maiornrconta = $sqlmaxconta[0]['MAXNRPARCREC'];

                $filialconta = $conta['CDFILIAL'];
                $nrlanctonf  = $conta['NRLANCTONF'];
                $nrseqvenrec = $conta['NRSEQVENREC'];

                $vrconta = round($parcelas[$i]['VRFINALBREC'] * $conta['VRATUAREC']/ $vrantboleto, 2);
                $vrtotconta+=$vrconta;


                //Atualiza fluxo de caixa para a conta
                FluxodeCaixa::atualizaFluxo($conta['CDEMPRESA'],
                    $conta['CDFIMOVFLREC'],
                    $conta['CDCLASSFINA'],
                    $parcelas[$i]['DTVENBOLREC'],
                    $vrconta,
                    'E',
                    'Entrada', $nrorg);
                //Atualiza as contas que estÃ£o ligadas ao boleto parcelado
                if($i==0){
                    $nrparcelarec = $conta['NRPARCELAREC'];


                    FluxodeCaixa::atualizaFluxo($conta['CDEMPRESA'],
                        $conta['CDFIMOVFLREC'],
                        $conta['CDCLASSFINA'],
                        $conta['DTMOVFLUXO'],
                        $conta['VRATUAREC'],
                        $conta['IDMOVFLUXO'],
                        'Entrada', $nrorg);

                    /** @var $contareceber Contareceber */
                    $contareceber = Contareceber::
                    findOneByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg(
                        $filialconta,$nrlanctonf,$nrseqvenrec,$nrparcelarec,$nrorg);
                    $contareceber->setVratuarec($vrconta);
                    $contareceber->setVrorigrec(round($vrconta - $conta['VRAJUSTE'], 2));
                    $contareceber->setDtatuavenrec($dtvenbolrec);
                    $contareceber->setIdmovfluxo('E');
                    $contareceber->setCdmotiparc($parcelas[$i]['CDMOTIPARC']);
                    $contareceber->setNrparcelarec($nrparcelarec);
                    $contareceber->persist();
                }
                else{
                    $nrparcelarec = str_pad($maiornrconta+1,2,'0',STR_PAD_LEFT);

                    $novaconta = new Contareceber();
                    $novaconta->setCdfilial($filialconta);
                    $novaconta->setNrlanctonf($nrlanctonf);
                    $novaconta->setNrseqvenrec($nrseqvenrec);
                    $novaconta->setNrparcelarec($nrparcelarec);
                    $novaconta->setCdfilibolrec($cdfilial);
                    $novaconta->setNrlanbolrec($nrnovoboleto);
                    $novaconta->setVratuarec($vrconta);
                    $novaconta->setCdmotiparc(($parcelas[$i]['CDMOTIPARC']));
                    $novaconta->setVrorigrec($vrconta);
                    $novaconta->setDtatuavenrec($dtvenbolrec);
                    $novaconta->setCdempresa($conta['CDEMPRESA']);
                    $novaconta->setCdfimovflrec($conta['CDFIMOVFLREC']);
                    $novaconta->setCdclassfina($conta['CDCLASSFINA']);
                    $novaconta->setDtmovfluxo($dtvenbolrec);
                    $novaconta->setIdmovfluxo('E');
                    $novaconta->setQtdiasvenrec($conta['QTDIASVENREC']);
                    $novaconta->setIdstatrec($conta['IDSTATREC']);
                    $novaconta->setCdtpctreceb($conta['CDTPCTRECEB']);
                    $novaconta->setDtorigvenrec($dtvenbolrec);
                    $novaconta->setDscontarec($conta['DSCONTAREC']);
                    $novaconta->setCdfilialdup($conta['CDFILIALDUP']);
                    $novaconta->setNrlanctodup($conta['NRLANCTODUP']);
                    $novaconta->setNrorg($nrorg);
                    $novaconta->setIdcontaimposto('N');
                    $novaconta->setDtinclusao(DateUtil::getDataAtual());
                    $novaconta->persist();

                }

                //LanÃ§a a diferenÃ§a entre a soma das contas e o valor do boleto na Ãºltima conta
                if($contadorcontas == count($contasoriginais)){
                    if ((float) $vrtotconta != (float)$parcelas[$i]['VRFINALBREC'])
                        $vrconta +=  $parcelas[$i]['VRFINALBREC'] - $vrtotconta;
                }

                if ($i == 0){
                    $contareceber->setVratuarec($vrconta);
                    $contareceber->persist();
                }
                else{
                    $novaconta->setVratuarec($vrconta);
                    $novaconta->setVrorigrec($vrconta);
                    $novaconta->persist();
                    \Setforms2\Shared\Registry::getEntityManager()->flush();
                }

                $totalrateio = 0;
                //Apaga o rateio das contas originais e lanÃ§a-os novamente
                /** @var $delrateio Ratctreceber */
                $delrateio = Ratctreceber::findByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg(
                    $filialconta, $nrlanctonf, $nrseqvenrec,
                    $nrparcelarec, $nrorg
                );

                $rateios = Ratctreceber::findByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg(
                    $filialconta, $nrlanctonf, $nrseqvenrec,
                    $nrparcelarec, $nrorg
                );

                if((bool)$delrateio === true){
                    /** @var $rat Ratctreceber*/
                    foreach ($delrateio as $rat)
                        $rat->delete();
                }
                \Setforms2\Shared\Registry::getEntityManager()->flush();

                $contadorrateio = 0;
                /** @var $rateio Ratctreceber */
                foreach ($rateios as $rateio){
                    $contadorrateio++;
                    $vrrateio = round($vrconta * ((float)$rateio->getVrratccctrec() / (float)($conta['VRATUAREC'])) ,2);
                    $totalrateio += $vrrateio;

                    $novorateio = new Ratctreceber();
                    $novorateio->setCdfilial($filialconta);
                    $novorateio->setNrlanctonf($nrlanctonf);
                    $novorateio->setNrseqvenrec($nrseqvenrec);
                    $novorateio->setNrparcelarec($nrparcelarec);
                    $novorateio->setCdcentcust($rateio->getCdcentcust());
                    $novorateio->setCdfiratctrec($rateio->getCdfiratctrec());
                    $novorateio->setVrratccctrec($vrrateio);
                    $novorateio->setDtinclusao(DateUtil::getDataAtual());
                    $novorateio->setNrorg($nrorg);

                    //Se existir qualquer diferenÃ§a no valor do rateio e da conta
                    if($contadorrateio == count($rateios)){
                        if((float)$vrtotconta != (float)$vrconta)
                            $novorateio->setVrratccctrec($vrrateio-($vrconta-$totalrateio));
                    }
                    $novorateio->persist();

                }
            }
        }
    }
    public static function DeletaParcela(\Financeiro\Entities\Boletreceber $boletoant,
                                         \Financeiro\Entities\Boletreceber $boletoatual,
                                         $qtdparcelas)
    {
        try{
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            $rep = \Financeiro\Entities\Boletreceber::getRepository();
            /** @var $tprecboleto Tprecboleto */
            $tprecboleto = Tprecboleto::findOneByCdfilialAndNrlanbolrecAndNrorg($boletoatual->getCdfilial(),
                $boletoatual->getNrlanbolrec(),
                $boletoatual->getNrorg());
            if ((bool) $tprecboleto === true){
                $tprecboleto->delete();
                $tprecboleto->persist();
            }

            //Updata o a parcela anterior com o valor do boleto a ser excluido
            $boletoant->setVrbolrec(round($boletoant->getVrbolrec(), 2) + round($boletoatual->getVrbolrec(), 2));
            $boletoant->setVrfinalbrec(round($boletoant->getVrfinalbrec(), 2)+round($boletoatual->getVrfinalbrec(), 2));
            $boletoant->setDslinhabarrec(null);
            $boletoant->setCdnosnumrec(null);
            $boletoant->setCdbarrarec(null);

            if($qtdparcelas == 2){
                $boletoant->setCdmotiparc('');
                $boletoant->setNrparcbolrec('00');
            }
            $boletoant->persist();

            //Busca os ajustes da parcela a ser excluÃ­da
            $ajustes = Ajbolrec::findByCdfilialAndNrlanbolrecAndNrorg($boletoatual->getCdfilial(),
                $boletoatual->getNrlanbolrec(),
                $boletoatual->getNrorg());

            /** @var $ajuste Ajbolrec */
            foreach ($ajustes as $ajuste){
                /** @var $ajustebolant Ajbolrec */
                $ajustebolant = Ajbolrec::findOneByCdfilialAndNrlanbolrecAndCdajuctrecAndNrorg
                ($boletoant->getCdfilial(),$boletoant->getNrlanbolrec(),
                    $ajuste->getCdajuctrec(),$boletoant->getNrorg());

                //se ja existir no boleto anteior o cÃ³digo de ajuste do boleto excluido updata o mesmo
                if((bool) $ajustebolant === true){
                    $ajustebolant->setVrajbolrec(round($ajustebolant->getVrajbolrec(), 2)+
                        round($ajuste->getVrajbolrec(), 2));
                    $ajustebolant->persist();
                }else{
                    //se nÃ£o existir insere
                    $novoajuste = new Ajbolrec();
                    $novoajuste->setNrorg($ajuste->getCdajuctrec());
                    $novoajuste->setCdfilial($boletoant->getCdfilial());
                    $novoajuste->setNrlanbolrec($boletoant->getNrlanbolrec());
                    $novoajuste->setVrajbolrec(round($ajuste->getVrajbolrec(), 2));
                    $novoajuste->setDtinclusao(DateUtil::getDataAtual());
                    $novoajuste->persist();
                }

                //Busca existÃªncia de adiantamento para a parcela ant.
                /** @var $adiantamentoAnt \Financeiro\Entities\Adiantareceb */
                $adiantamentoAnt =  \Financeiro\Entities\Adiantareceb::
                findOneByCdfiliajbolrAndNrlanbolrecCdajuctrecAndNrorg
                ($boletoant->getCdfilial(),$boletoant->getNrlanbolrec(),
                    $ajuste->getCdajuctrec(),$boletoant->getNrorg());

                //Busca existÃªncia de devolucao para a parcela ant.
                /** @var $devolucaoAnt Devolucaorec */
                $devolucaoAnt = Devolucaorec:: findOneByCdfiliajbolrAndNrlanbolrecCdajuctrecAndNrorg
                ($boletoant->getCdfilial(),$boletoant->getNrlanbolrec(),
                    $ajuste->getCdajuctrec(),$boletoant->getNrorg());

                //Busca existÃªncia de adiantamento para a parcela atual.
                /** @var $adiantamentoAtual \Financeiro\Entities\Adiantareceb */
                $adiantamentoAtual = \Financeiro\Entities\Adiantareceb::
                findOneByCdfiliajbolrAndNrlanbolrecCdajuctrecAndNrorg
                ($ajuste->getCdfilial(),$ajuste->getNrlanbolrec(),
                    $ajuste->getCdajuctrec(),$ajuste->getNrorg());

                //Busca existÃªncia de devolucao para a parcela atual.
                /** @var $devolucaoAtual Devolucaorec */
                $devolucaoAtual = Devolucaorec:: findOneByCdfiliajbolrAndNrlanbolrecCdajuctrecAndNrorg
                ($ajuste->getCdfilial(),$ajuste->getNrlanbolrec(),
                    $ajuste->getCdajuctrec(),$ajuste->getNrorg());
                $ajuste->delete();
                \Setforms2\Shared\Registry::getEntityManager()->flush();


                if ((bool) $adiantamentoAtual === true){

                    if((bool)$adiantamentoAnt === true){
                        $adiantamentoAnt->setVradiantarec(round($adiantamentoAnt->getVradiantarec(), 2)+
                            round($adiantamentoAtual->getVradiantarec(), 2));
                        $adiantamentoAnt->persist();

                    }
                    else{
                        $novoadiantamento = new \Financeiro\Entities\Adiantareceb();
                        $novoadiantamento->setNrorg($adiantamentoAtual->getNrorg());
                        $novoadiantamento->setCdfilial($adiantamentoAtual->getCdfilial());
                        $novoadiantamento->setNrlanctonf($adiantamentoAtual->getNrlanctonf());
                        $novoadiantamento->setNrseqvenrec($adiantamentoAtual->getNrseqvenrec());
                        $novoadiantamento->setNrparcelarec($adiantamentoAtual->getNrparcelarec());
                        $novoadiantamento->setCdfiliajbolr($boletoant->getCdfilial());
                        $novoadiantamento->setNrlanbolrec($boletoant->getNrlanbolrec());
                        $novoadiantamento->setCdajuctrec($adiantamentoAtual->getCdajuctrec());
                        $novoadiantamento->setDtinclusao(DateUtil::getDataAtual());
                        $novoadiantamento->persist();
                    }
                }
                if((bool)$devolucaoAtual === true){
                    $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg
                    ($devolucaoAtual->getCdfilial(),$devolucaoAtual->getNrlanctonf(),
                        $devolucaoAtual->getNrorg());

                    if((bool) $devolucaoAnt === true){
                        /** @var $notafiscal Notafiscal */

                        $notafiscal->setVrdevolnfutl($notafiscal->getVrdevolnfutl()+
                            $devolucaoAtual->getVrdevolucaorec());
                        $notafiscal->setCdfilial($devolucaoAtual->getCdfilial());
                        $notafiscal->setNrlanctonf($devolucaoAtual->getNrlanctonf());
                        $notafiscal->persist();

                        $devolucaoAnt->setVrdevolucaorec(round($devolucaoAnt->getVrdevolucaorec() + $devolucaoAtual->getVrdevolucaorec(), 2));
                        $devolucaoAnt->persist();

                    }else{
                        $notafiscal->setVrdevolnfutl($notafiscal->getVrdevolnfutl()+
                            $devolucaoAtual->getVrdevolucaorec());
                        $notafiscal->setCdfilial($devolucaoAtual->getCdfilial());
                        $notafiscal->setNrlanctonf($devolucaoAtual->getNrlanctonf());
                        $notafiscal->persist();

                        $novadevolucao = new Devolucaorec();
                        $novadevolucao->setNrorg($boletoant->getNrorg());
                        $novadevolucao->setNrlanbolrec($boletoant->getNrlanbolrec());
                        $novadevolucao->setCdajuctrec($devolucaoAtual->getCdajuctrec());
                        $novadevolucao->setCdfilial($devolucaoAtual->getCdfilial());
                        $novadevolucao->setNrlanctonf($devolucaoAtual->getNrlanctonf());
                        $novadevolucao->setVrdevolucaorec($devolucaoAtual->getVrdevolucaorec());
                        $novadevolucao->setDtinclusao(DateUtil::getDataAtual());
                        $novadevolucao->persist();
                    }

                }
            }
            //Busca contas do boleto excluÃ­do
            $contasreceber = Contareceber::findByCdfilibolrecAndNrlanbolrecAndNrorg($boletoatual->getCdfilial(),
                $boletoatual->getNrlanbolrec(),
                $boletoatual->getNrorg());
            $primconta = true;
            $countconta = 0;
            $contasoriginais = null;
            /** @var $conta Contareceber */
            foreach ($contasreceber as $conta){

                if ($primconta){
                    $contasoriginais = $rep->getContaOriginalParcelamento($boletoant->getNrorg(),
                        $boletoant->getNrlanbolrec(),
                        $boletoant->getCdfilial());

                    $primconta = false;
                }

                $ajustesconta = $rep->getAjusteConta($conta->getCdfilial(),
                    $conta->getNrlanbolrec(),
                    $conta->getNrorg());

                foreach($ajustesconta as $ajusteconta){
                    $ajctreceber = Ajctreceber::
                    findOneByNrorgAndCdfilialAndAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndCdajuctrecAndDtajusterec
                    ($ajusteconta['NRORG'],$ajusteconta['CDFILIAL'],$ajusteconta['NRLANCTONF'],
                        $ajusteconta['NRSEQVENREC'],$contasoriginais[$countconta]['NRPARCELAREC'],
                        $ajusteconta['CDAJUCTREC'],$ajusteconta['DTAJUSTEREC']);

                    if((bool)$ajctreceber === false){
                        $novoajuste = new Ajctreceber();
                        $novoajuste->setCdfilial($ajusteconta['CDFILIAL']);
                        $novoajuste->setNrlanctonf($ajusteconta['NRLANCTONF']);
                        $novoajuste->setDtajusterec($ajusteconta['DTAJUSTEREC']);
                        $novoajuste->setCdajuctrec($ajusteconta['CDAJUCTREC']);
                        $novoajuste->setDsajusterec($ajusteconta['DSAJUSTEREC']);
                        $novoajuste->setNrseqvenrec($ajusteconta['NRSEQVENREC']);
                        $novoajuste->setNrparcelarec($contasoriginais[$countconta]['NRPARCELAREC']);
                        $novoajuste->setCdclassfina($ajusteconta['CDCLASSFINA']);
                        $novoajuste->setCdemmfajcntr($ajusteconta['CDEMMFAJCNTR']);
                        $novoajuste->setCdfimfajcntr($ajusteconta['CDFIMFAJCNTR']);
                        $novoajuste->setDtmovfluxo($ajusteconta['DTMOVFLUXO']);
                        $novoajuste->setIdmovfluxo($ajusteconta['IDMOVFLUXO']);
                        $novoajuste->setVrajusterec($ajusteconta['VRAJUSTEREC']);
                        $novoajuste->setDtinclusao(DateUtil::getDataAtual());
                        $novoajuste->persist();
                    }

                    $delajctreceber = Ajctreceber::
                    findOneByNrorgAndCdfilialAndAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndCdajuctrecAndDtajusterec
                    ($ajusteconta['NRORG'],$ajusteconta['CDFILIAL'],$ajusteconta['NRLANCTONF'],
                        $ajusteconta['NRSEQVENREC'],$ajusteconta['NRPARCELAREC'],
                        $ajusteconta['CDAJUCTREC'],$ajusteconta['DTAJUSTEREC']);

                    $delajctreceber->delete();
                }

                $contaoriginal = Contareceber::findOneByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg(
                    $contasoriginais[$countconta]['CDFILIAL'], $contasoriginais[$countconta]['NRLANCTONF'], $contasoriginais[$countconta]['NRSEQVENREC'], $contasoriginais[$countconta]['NRPARCELAREC'], $contasoriginais[$countconta]['NRORG']
                );

                $valorContaOld = round($contasoriginais[$countconta]['VRATUAREC'], 2);

                $contaoriginal->setVratuarec(round($contasoriginais[$countconta]['VRATUAREC'], 2) +
                    round($conta->getvratuarec(), 2));
                $contaoriginal->setVrorigrec(round($contasoriginais[$countconta]['VRORIGREC'], 2) +
                    round($conta->getvrorigrec(), 2));

                $contaoriginal->persist();

                self::RateiaConta($contasoriginais[$countconta]['CDFILIAL'],$contasoriginais[$countconta]['NRLANCTONF'],
                    $contasoriginais[$countconta]['NRSEQVENREC'],$contasoriginais[$countconta]['NRPARCELAREC'],
                    round($contasoriginais[$countconta]['VRATUAREC'], 2) + round($conta->getVratuarec(), 2),
                    $valorContaOld,$contasoriginais[$countconta]['NRORG']);
                $delconta  = Contareceber::findOneByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg
                ($conta->getCdfilial(),$conta->getNrlanctonf(),$conta->getNrseqvenrec(),
                    $conta->getNrparcelarec(), $conta->getNrorg());
                $delconta->delete();


            }

            \Setforms2\Shared\Registry::getEntityManager()->flush();
            // Exclui parcela
            $boletoatual->delete();


            $connection->flush();
            $connection->commit();
        }
        catch(\Exception $e){
            $connection->rollback();
            throw $e;

        }

    }
    private static function RateiaConta($cdfilial,$nrlanctonf,$nrseqvenrec,$nrparcelarec, $valornovo, $valorantigo,$nrorg){
        try {
            $rep = \Financeiro\Entities\Boletreceber::getRepository();

            $rateios = $rep->getValorRateioConta($cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $nrorg);
            $totalrateio = 0;
            $delrateio = Ratctreceber::
            findByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg($cdfilial,
                $nrlanctonf,
                $nrseqvenrec,
                $nrparcelarec,
                $nrorg);
            if ((bool) $delrateio === true){
                /** @var $rat Ratctreceber*/
                foreach ($delrateio as $rat)
                    $rat->delete();
            }

            \Setforms2\Shared\Registry::getEntityManager()->flush();

            $contadorrateio = 0;
            foreach ($rateios as $rateio) {
                $contadorrateio++;
                $valorrateio = round($valornovo * ($rateio['VRRATCCCTREC'] / $valorantigo), 2);
                $totalrateio += $valorrateio;

                $novorateio = new Ratctreceber();
                $novorateio->setCdfilial($cdfilial);
                $novorateio->setNrlanctonf($nrlanctonf);
                $novorateio->setNrseqvenrec($nrseqvenrec);
                $novorateio->setNrparcelarec($nrparcelarec);
                $novorateio->setCdcentcust($rateio['CDCENTCUST']);
                $novorateio->setCdfiratctrec($rateio['CDFIRATCTREC']);
                $novorateio->setVrratccctrec($valorrateio);
                $novorateio->setDtinclusao(DateUtil::getDataAtual());
                $novorateio->setNrorg($nrorg);

                //Se existir qualquer diferenÃ§a no valor do rateio e da conta
                if ($contadorrateio == count($rateios)) {
                    if ((float)$totalrateio != (float)$valornovo)
                        $novorateio->setVrratccctrec($valorrateio - ($valornovo - $totalrateio));
                }
                $novorateio->persist();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
}